{% extends "ishtar/sheet.html" %}
{% load i18n window_field window_tables %}

{% block header_title %}<h1>{% trans "Person"%}</h1>{% endblock %}
{% block toolbar %}{% include "ishtar/sheet_toolbar.html" %}{% endblock %}

{% block content %}
<h3>{% trans "Identity" %}</h3>

{% field "Name" item.name %}
{% field "Surname" item.surname %}
{% field "Raw name" item.raw_name %}
{% field "Created by" item.history_creator.ishtaruser.full_label %}
{% field "Email" item.email %}
{% field "Type(s)" item.person_types_list %}

{% if item.phone or item.phone2 or item.phone3 or item.mobile_phone %}
<table class='formset table-form'>
  <caption>{% trans "Phone" %}</caption>
  <tr>
    <th>{% trans "Phone type" %}</th>
    <th>{% trans "Number" %}</th>
  </tr>
  {% if item.phone or item.phone_desc %}<tr>
    <td>{% if item.phone_desc %}{{item.phone_desc}}{% else %}&ndash;{% endif %}</td>
    <td>{% if item.phone %}{{item.phone}}{% else %}&ndash;{% endif %}</td>
  </tr>{% endif %}
  {% if item.phone2 or item.phone_desc2 %}<tr>
    <td>{% if item.phone_desc2 %}{{item.phone_desc2}}{% else %}&ndash;{% endif %}</td>
    <td>{% if item.phone2 %}{{item.phone2}}{% else %}&ndash;{% endif %}</td>
  </tr>{% endif %}
  {% if item.phone3 or item.phone_desc3 %}<tr>
    <td>{% if item.phone_desc3 %}{{item.phone_desc3}}{% else %}&ndash;{% endif %}</td>
    <td>{% if item.phone3 %}{{item.phone3}}{% else %}&ndash;{% endif %}</td>
  </tr>{% endif %}
  {% if item.mobile_phone %}<tr>
    <td>{% trans "Mobile phone" %}</td>
    <td>{{item.mobile_phone}}</td>
  </tr>{% endif %}
</table>
{% endif %}

{% if item.address or item.postal_code or item.town %}
<h3>{% trans "Business address" %}</h3>
{% field "Address" item.address %}
{% field "Address complement" item.address_complement %}
{% field "Postal code" item.postal_code %}
{% field "Town" item.town %}
{% endif %}

{% if item.alt_address or item.alt_postal_code or item.alt_town %}
<h3>{% trans "Other address" %}</h3>
{% field "Address" item.alt_address %}
{% field "Address complement" item.alt_address_complement %}
{% field "Postal code" item.alt_postal_code %}
{% field "Town" item.alt_town %}
{% endif %}


{% if item.attached_to %}<h3>{% trans "Associated organization"%}</h3>
{% field "Name" item.attached_to %}
{% field "Address" item.attached_to.address %}
{% field "Address complement" item.attached_to.address_complement %}
{% field "Postal code" item.attached_to.postal_code %}
{% field "Town" item.attached_to.town %}
{% field "Phone" item.attached_to.phone %}
{% field "Mobile phone" item.attached_to.mobile_phone %}
{% endif %}


{% trans "Associated operations as scientist" as ao %}
{% if item.operation_scientist_responsability.count %}
{% dynamic_table_document ao 'operations' 'scientist' item.pk '' output %}
{% endif %}
{% trans "Associated operations as responsible" as ao %}
{% if item.operation_responsability.count %}
{% dynamic_table_document ao 'operations' 'in_charge' item.pk '' output %}
{% endif %}


{% trans "Associated archaelogical files" as af %}
{% if item.file_responsability.count %}
{% dynamic_table_document af 'files' 'in_charge' item.pk '' output %}
{% endif %}

<table>
  <caption>{%trans "Documents"%}</caption>
  <tr>
    <th>{% trans "Year" %}</th>
    <th>{% trans "Title" %}</th>
    <th>{% trans "Type" %}</th>
    <th>{% trans "Link" %}</th>
  </tr>
  {% for author in item.author.all %}
  {% for doc in author.related_sources %}
  <tr>
    <td class='string'>{{ doc.creation_date|date:"Y"}}</td>
    <td class='string'>{{ doc.title }}</td>
    <td class='string'>{{ doc.source_type }}</td>
    <td class='string'>{% if doc.associated_url  %}<a href='{{doc.associated_url}}'>{% trans "Link"%}</a>{% endif %}</td>
  </tr>
  {% endfor %}
  {% empty %}
  <tr><td colspan="4" class='no_items'>{% trans "No document associated to this person" %}</td></tr>
  {% endfor %}
</table>


{% endblock %}
