{% load i18n %}
<table class='simple'>
  <caption>{{caption}}</caption>
  <tr>
    <th>{% trans "Ref." %}</th>
    <th>{% trans "Type" %}</th>
    <th>{% trans "Date" %}</th>
    <th>{% trans "Object" %}</th>
  </tr>
  {% for act in data %}
  <tr>
    <td>{{act.full_ref}}</td>
    <td class='string'>{{act.act_type}}</td>
    <td>{{act.signature_date}}</td>
    <td>{{act.act_object}}</td>
  </tr>
  {% empty %}
  <tr><td colspan="4" class='no_items'>{% trans "No administrative act associated" %}</td></tr>
  {% endfor %}
</table>
