{% load i18n %}
  {% if form.non_field_errors %}<tr class='errors'>
    <td colspan='3'>{{form.non_field_errors}}</td>
  </tr>{%endif%}
  {% for field in form %}{% if not field.is_hidden %}
  <tr{% if field.field.required %} class='required'{% endif %}>
    <th>{{ field.label_tag }}</th>
    <td> {{ field.errors }}{{field|safe}}</td>{% if field.help_text %}
    <td><a href="#{{field.auto_id}}" class="help_display" title="{% trans "Help"%}">?</a></td>
  </tr>
  <tr class="help_text" id="{{field.auto_id}}_help">
    <td colspan="3"><div>{{field.help_text}}</div></td>
  {%endif%}</tr>{% else %}{{field}}{% endif %}{% endfor %}
