{% extends "ishtar/wizard/default_wizard.html" %}
{% load i18n range table_form %}
{% block wizard_form %}
<form action="." method="post" name='wizard'{% if wizard.form.file_upload %} enctype="multipart/form-data"{% endif %}>{% csrf_token %}
<div class='form'>
{% if wizard.form.media %}{{ wizard.form.media }}{% endif %}
{{ wizard.management_form }}

<table>
  {% if wizard.form.non_field_errors %}<tr class='errors'>
    <td colspan='3'>{{wizard.form.non_field_errors}}</td>
  </tr>{%endif%}

  <tr>
    <th>{{ wizard.form.planning_service.label_tag }}</th>
    <td> {{ wizard.form.planning_service.errors }}{{wizard.form.planning_service|safe}}</td>
  </tr>
  <tr>
    <th>{{ wizard.form.responsible_town_planning_service.label_tag }}</th>
    <td> {{ wizard.form.responsible_town_planning_service.errors }}{{wizard.form.responsible_town_planning_service|safe}}</td>
  </tr>
</table>


<div>
<table>
  {% if permit_type %}
  <tr>
    <th colspan='3'>{{permit_type}}</th>
  </tr>{% endif %}
  <tr>
    <th>{{ wizard.form.permit_reference.label_tag }}{% if permit_type_code %} [{{permit_type_code}}]{% endif %}</th>
    <td> {{ wizard.form.permit_reference.errors }}{{wizard.form.permit_reference|safe}}</td>
  </tr>
</table>
</div>

<input type="hidden" name="{{ step_field }}" value="{{ step0 }}" />
{{ previous_fields|safe }}
{% block "validation_bar" %}
{% include 'ishtar/wizard/validation_bar.html' %}
{% endblock %}
</div>
</form>
{% endblock %}



