{% load i18n %}
{% if data.count %}<p><label>{% trans caption %}{% trans ":"%}</label>
    <span class='value'>{% for d in data.all %}
        {% if forloop.counter0 %}, {% endif %}{{ d }}
    {% endfor %}</span>
</p>
{% endif %}

