{% load i18n %}
<table class='simple'>
  <caption>{{ parcels_label }}</caption>
  <tr>
    <th>{% trans "Town" %}</th>
    <th>{% trans "Year" %}</th>
    <th>{% trans "Section" %}</th>
    <th>{% trans "Parcels" %}</th>
    {#<th>{% trans "Owner" %}</th>#}
  </tr>
  {% for parcel in item.grouped_parcels %}
  <tr>
    <td class='string'>{{parcel.town}}</td>
    <td>{% if parcel.year %}{{parcel.year}}{% endif %}</td>
    <td>{{parcel.section}}</td>
    <td>{{parcel.parcel_numbers|join:", "}}</td>
    {#<td class='string'>{{operation.parcel.owner}}</td>#}
  </tr>
  {% empty %}
  <tr><td colspan="4" class='no_items'>{% trans "No parcel" %}</td></tr>
  {% endfor %}
</table>
