{% load i18n %}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">

<head>
    <link rel="shortcut icon" href="{{STATIC_URL}}/media/images/favicon.png">
    <title>{% block title %}Ishtar{% if APP_NAME %} - {{APP_NAME}}{%endif%}{% endblock %}
    </title>
    <link rel="stylesheet" href="{{STATIC_URL}}/media/style.css" />
</head>
<body>
<div>
  <h3>{% trans "Page not found" %}</h3>
  <p><a href='/'>{% trans "Back to main page" %}</a></p>
</div>
</body>

</html>

