{% load i18n %}
<table class='simple'>
  <caption>{{caption}}</caption>
  <tr>
    <th>{% trans "Title" %}</th>
    <th>{% trans "Type" %}</th>
    <th>{% trans "Authors" %}</th>
    <th>{% trans "Related to" %}</th>
    <th>{% trans "Link" %}</th>
  </tr>
  {% for doc in data %}
  <tr>
    <td class='string'>{{ doc.title }}</td>
    <td class='string'>{{doc.source_type}}</td>
    <td class='string'>{{ doc.authors.all|join:", " }}</td>
    <td class='string'><a href="#" onclick='load_window("{{doc.owner.show_url}}")'>{{doc.owner.short_label}}</a></td>
    <td class='string'>{% if doc.associated_url  %}<a href='{{doc.associated_url}}'>{% trans "Link"%}</a>{% endif %}</td>
  </tr>
  {% endfor %}
</table>

