{% load i18n %}
<table>
<tr>{% for item in basket.items.all %}
    <td><a class="display_details" href="#" onclick="load_window('{{item_url}}/{{item.pk}}/');">{% trans 'Details' %}</a></td>
    <td>{{item.full_label}}</td>
    <td><a class='async-link' data-target='#basket-content' href='{{delete_url}}/{{basket.pk}}/{{item.pk}}/'>{% trans "remove" %}</a></td></tr>{% endfor %}
</table>
<script type='text/javascript'>
    $("a.async-link").click(manage_async_link);
</script>
