{% load i18n %}
<div class='dashboard'>
  <div>
<script language="javascript" type="text/javascript">
$(document).ready(function(){
var values_users = [];
{% for user_type in ishtar_users.types %}
values_users.push(['{{user_type.person__person_types__label}}', {{user_type.number}}]); {% endfor%}

var plot_users = jQuery.jqplot ('user_chart', [values_users],
    {
      seriesDefaults: {
        renderer: jQuery.jqplot.PieRenderer,
        rendererOptions: {
          showDataLabels: true
        }
      },
      legend: { show:true, location: 'e' }
    }
 );
});
</script>
    <div id="user_chart" style="height:300px; width:700px;"></div>
    <div class='table'>
    <table class='resume'>
      <tr><th>{% trans "User type" %}</th><th>{% trans "Number" %}</th></tr>
    {% for user_type in ishtar_users.types %}
      <tr>
        <td class='string'>{{user_type.person__person_types__label}}{#TODO: Display all#}</td>
        <td>{{user_type.number}}</td>
      </tr>
    {% endfor%}
    </table>
    </div>
</div>

