{% extends "ishtar/sheet_contextrecord.html" %}
{% block header %}
<link rel="stylesheet" href="{{STATIC_URL}}/media/style_basic.css" />
{% endblock %}
{% block main_head %}
{{ block.super }}
<div id="pdfheader">
Ishtar &ndash; {{APP_NAME}} &ndash; {{item}}
</div>
{% endblock %}
{%block head_sheet%}{%endblock%}
{%block main_foot%}
<div id="pdffooter">
&ndash; <pdf:pagenumber/> &ndash;
</div>
</body>
</html>
{%endblock%}
