{% extends "ishtar/sheet_source.html" %}
{% load i18n window_field link_to_window %}

{% block head_sheet %}
{{block.super}}
<h1>{% trans "Context record source"%}</h1>
{% endblock %}

{% block related %}
{% field "Related context record" item.owner '' item.owner|link_to_window %}
{% endblock %}
