{% load i18n %}
{% if link %}<p><label>{% trans caption %}{% trans ":"%}</label>
<span class='value'><a target="_blank" href='{{link|safe}}'>{% if link_name %}{{link_name}}{% else %}{% trans "link" %}{% endif %}</a></span></p>{% endif%}
