{% extends "base.html" %}
{% load i18n inline_formset %}
{% block content %}
<h2>{{page_name}}</h2>
<div class='form' id='global-vars'>
<form action="." method="post">{% csrf_token %}
  {% inline_formset ' ' formset %}
<input type="submit" value="{% trans "Validate" %}"/>
</form>
</div>
{% endblock %}
