{% extends "admin/change_form.html" %}
{% load i18n %}

{% block breadcrumbs %}
	<div class="breadcrumbs">
		<a href="{% url admin:index %}">{% trans "Home" %}</a> &rsaquo;
		<a href="{% url admin:app_list app_label %}">{{app_label|capfirst|escape}}</a> &rsaquo;
		<a href="{{changelist_url}}">{{opts.verbose_name_plural|capfirst}}</a> &rsaquo;
		<a href="{{change_url}}">{{original|truncatewords:"18"}}</a> &rsaquo;
		<a href="../">{% trans "History" %}</a> &rsaquo;
		{% blocktrans with original_opts.verbose_name as verbose_name %}Revert {{verbose_name}}{% endblocktrans %}
	</div>
{% endblock %}

{% comment %}Hack to remove "Save as New" and "Save and Continue" buttons {% endcomment %}
{% block content %}
	{% with 1 as is_popup %}
		{{block.super}}
	{% endwith %}
{% endblock %}

{% block form_top %}
	<p>{% blocktrans %}Press the save button below to revert to this version of the object.{% endblocktrans %}</p>
{% endblock %}
