# force the retranslation of generated strings and external module
from django.utils.translation import ugettext as _
_(u"username")
_(u"email address")
_(u"warehouse")
_(u"New warehouse")
_(u"warehouse")
_(u"New organization")
_(u"New person")
_(u"New author")
