{% load i18n  %}
<html>
<body>
<div class='form'>
<form enctype="multipart/form-data" action="." method="post">{% csrf_token %}
{{form}}
<input type="submit" value="{% trans "Validate" %}"/>
</form>
</div>
</body>
</html>
