{% extends "ishtar/sheet.html" %}
{% load i18n window_field link_to_window %}
{% block toolbar %}{% include "ishtar/sheet_toolbar.html" %}{% endblock %}
{% block content %}
{% block general %}
{% field "Title" item.title %}
{% field "Source type" item.source_type %}
{% field "Format type" item.format_type %}
{% field "Scale" item.scale %}
{% field_url "Web link" item.associated_url %}
{% field_multiple "Authors" item.authors %}
{% field "Item number" item.item_number %}
{% field "Ref." item.reference %}
{% field "Internal ref." item.internal_reference %}
{% field "Creation date" item.creation_date %}
{% field "Receipt date" item.receipt_date %}
{% field "Receipt date in documentation" item.receipt_date_in_documentation %}
{% field "Has a duplicate" item.duplicate %}
{% field "Description" item.description %}
{% field "Comment" item.comment %}
{% field "Additional information" item.additional_information %}
{% endblock %}
{% block related %}
{% field "Related item" item.owner %}
{% endblock %}
{% endblock %}

