{% extends "ishtar/wizard/default_wizard.html" %}
{% load i18n range table_form %}
{% block wizard_form %}
<form action="." method="post" name='wizard'{% if wizard.form.file_upload %} enctype="multipart/form-data"{% endif %}>{% csrf_token %}
<div class='form'>
{{ wizard.form.media }}
{{ wizard.management_form }}

<table>

<tr class='required'>
  <th><label for="id_preventiveplanning-{{CURRENT_ACTION}}-name">Nom de l'aménagement</label></th>
</tr>
<tr class='required'>
  <td>{{wizard.form.name|safe}}</td>
</tr>

</table>

<h4>Localisation</h4>
<table>

{% if wizard.form.main_town.errors %}<tr>
  <td colspan="2">{{wizard.form.main_town.errors}}</td>
</tr>{% endif %}

<tr>
  <th colspan='2'><label for="id_preventiveplanning-{{CURRENT_ACTION}}-towns">{% trans "Towns" %}</label></th>
</tr>
<tr>
  <td>{{wizard.form.town|safe}}</td>
</tr>

<tr>
  <th colspan='2'><label for="id_preventiveplanning-{{CURRENT_ACTION}}-department">{% trans "Departments" %}</label></th>
</tr>
<tr>
  <td>{{wizard.form.department|safe}}</td>
</tr>

<tr>
  <th colspan='2'><label for="id_preventiveplanning-{{CURRENT_ACTION}}-locality">Lieu-dit</label></th>
</tr>
<tr>
  <td colspan='2'>{{wizard.form.locality|safe}}</td>
</tr>

<tr>
  <th colspan='2'><label for="id_preventiveplanning-{{CURRENT_ACTION}}-address">Adresse</label></th>
</tr>
<tr>
  <td colspan='2'>{{wizard.form.address|safe}}</td>
</tr>

<tr>
  <th colspan='2'><label for="id_preventiveplanning-{{CURRENT_ACTION}}-postal_code">Code postal</label></th>
</tr>
<tr>
  <td colspan='2'>{{wizard.form.postal_code|safe}}</td>
</tr>

</table>

<h4>Surfaces</h4>
<table>

<tr>
  <th><label for="id_preventiveplanning-{{CURRENT_ACTION}}-total_surface">Surface totale des terrains</label></th>
</tr>
<tr>
  <td>{{wizard.form.total_surface|safe}}</td>
</tr>

<tr>
  <th><label for="id_preventiveplanning-{{CURRENT_ACTION}}-total_developed_surface">Surface totale aménagée</label></th>
</tr>
<tr>
  <td>{{wizard.form.total_developed_surface|safe}}</td>
</tr>

</table>

<input type="hidden" name="{{ step_field }}" value="{{ step0 }}" />
{{ previous_fields|safe }}
{% block "validation_bar" %}
{% include 'ishtar/wizard/validation_bar.html' %}
{% endblock %}
</div>
</form>
{% endblock %}
