{% load i18n %}
{% load url from future %}
<div id='validation-bar'>
  <input type="submit" id="submit_form" name='validate' value="{% trans "Validate" %}"/>
  {% if next_steps %}<input type="submit" id="submit_end_form" name='validate_and_end' value="{% trans "Validate and end" %}"/>{% endif %}
  <a href='{% url 'reset_wizards' %}' id="reset_wizards" class='button'>{% trans "Cancel" %}</a>
</div>
