{% extends "ishtar/sheet.html" %}
{% load i18n window_tables from_dict %}

{% block head_sheet %}
{{block.super}}
<h1>{% trans "Find"%}</h1>
{% endblock %}

{% block content %}

<div class='tool'>{%trans "Export as:"%} <a href='{% url show-findbasket item.pk "odt" %}'>{%trans "OpenOffice.org file"%}</a>, <a href='{% url show-findbasket item.pk "pdf" %}'>{%trans "PDF file"%}</a></div>
<div class='tool modify'><a href='{% url select_itemsinbasket item.pk %}'>{% trans "Modify" %}</a></div>

{% dynamic_table_document_large finds 'finds_for_ope' 'basket' item.pk 'TABLE_COLS_FOR_OPE' output %}

{% endblock %}
