.ui-checkbox-icon
{
    height:12px; display:inline-block;
    line-height:12px;
    font-size:12px;
    cursor:pointer;
    color:#707070;
    padding-left:18px;
    background-image: url(../images/checkbox_set-1.png);
    background-repeat:no-repeat;
    background-color:transparent;
}

 .ui-checkbox-icon-disbled
{
    height:12px; display:inline-block;
    line-height:12px;
    font-size:12px;
    cursor:default;
    color:#707070;
    padding-left:18px;
    background-image: url(../images/checkbox_set-1.png);
    background-repeat:no-repeat;
    background-color:transparent;
}

.ui-checkbox-icon:hover { color:#000; }
.ui-checkbox-selected
{
    background-position: 0 -48px;
}
.ui-checkbox-selected-over
{
    background-position: 0 -32px;
}
.ui-checkbox-unselected
{
    background-position: 0 0;
}
.ui-checkbox-unselected-over
{
    background-position: 0 -16px;
}


/* Checkbox On the Right */
 .ui-checkbox-right  
{
    padding-left: 0px;
    padding-right: 18px;
}

.ui-checkbox-selected-right
{
    background-position: right -48px;
}
.ui-checkbox-selected-over-right
{
    background-position: right -32px;
}
.ui-checkbox-unselected-right
{
    background-position: right 0;
}
.ui-checkbox-unselected-over-right
{
    background-position: right -16px;
}



.ui-sexyselect-modal-title-icon { background: url(../images/alert.png) no-repeat 0 0; display:inline-block; width:16px; height:16px;opacity:0.8;  }
.ui-sexyselect-styleize-label
{
    height:12px;
    line-height:12px;
    font-size:12px;
    cursor:pointer;
    color:#707070;
}