{% load i18n %}
<table class='simple'>
  <caption>{{caption}}</caption>
  <tr>
    <th>{% trans "Ref." %}</th>
    <th>{% trans "Name" %}</th>
    <th>{% trans "Periods" %}</th>
    <th>{% trans "Remains" %}</th>
  </tr>
  {% for archaeosite in data %}
  <tr>
    <td class='string'>{{archaeosite.reference}}</td>
    <td class='string'>{{archaeosite.name}}</td>
    <td class='string'>{% for period in archaeosite.periods.all %}{{period}}{% include "blocks/comma_list.html" %}{% endfor %}</td>
    <td class='string'>{% for remain in archaeosite.remains.all %}{{remain}}{% include "blocks/comma_list.html" %}{% endfor %}</td>
  </tr>
  {% empty %}
  <tr><td colspan="4" class='no_items'>{% trans "No archaeological site associated" %}</td></tr>
  {% endfor %}
</table>
