{% extends "ishtar/wizard/default_wizard.html" %}
{% load i18n range table_form %}
{% block form_detail %}

<table class='formset'>
  <caption>{% trans "Identity" %}</caption>
  {% table_field wizard.form.title %}
  {% table_field wizard.form.surname 1 %}
  {% table_field wizard.form.name 1 %}
  {% table_field wizard.form.raw_name %}
  {% table_field wizard.form.email %}
  {% table_field wizard.form.attached_to %}
</table>

<hr class='spacer'/>
<table class='formset table-form'>
  <caption>{% trans "Phone" %}</caption>
  <tr>
    <th>{% trans "Phone type" %}</th>
    <th>{% trans "Number" %}</th>
  </tr>
  <tr>
    <td>{{ wizard.form.phone_desc.errors }}{{wizard.form.phone_desc|safe}}</td>
    <td>{{ wizard.form.phone.errors }}{{wizard.form.phone|safe}}</td>
  </tr>
  <tr>
    <td>{{ wizard.form.phone_desc2.errors }}{{wizard.form.phone_desc2|safe}}</td>
    <td>{{ wizard.form.phone2.errors }}{{wizard.form.phone2|safe}}</td>
  </tr>
  <tr>
    <td>{{ wizard.form.phone_desc3.errors }}{{wizard.form.phone_desc3|safe}}</td>
    <td>{{ wizard.form.phone3.errors }}{{wizard.form.phone3|safe}}</td>
  </tr>
  <tr>
    <td>{% trans "Mobile phone" %}</td>
    <td>{{ wizard.form.mobile_phone.errors }}{{wizard.form.mobile_phone|safe}}</td>
  </tr>
</table>
<hr class='spacer'/>

<table class='formset'>
  <caption>{% trans "Business address" %}</caption>
  {% table_field wizard.form.address %}
  {% table_field wizard.form.address_complement %}
  {% table_field wizard.form.postal_code %}
  {% table_field wizard.form.town %}
  {% table_field wizard.form.country %}
</table>
<hr class='spacer'/>
<table class='formset'>
  <caption>{% trans "Other address" %}</caption>
  {% table_field wizard.form.alt_address 0 'Address' %}
  {% table_field wizard.form.alt_address_complement 0 'Address complement' %}
  {% table_field wizard.form.alt_postal_code 0 'Postal code' %}
  {% table_field wizard.form.alt_town 0 'Town' %}
  {% table_field wizard.form.alt_country 0 'Country' %}
</table>

{% endblock %}
